@@ -83,7 +83,8 @@ read.names<-function(nameFile,onlyScientific=TRUE){
 #' )
 #' tmpFile<-tempfile()
 #' writeLines(namesText,tmpFile)
-#' read.names.sql(tmpFile)
+#' sqlFile<-tempfile()
+#' read.names.sql(tmpFile,sqlFile)
 read.names.sql<-function(nameFile,sqlFile='nameNode.sqlite',overwrite=FALSE){
   if(file.exists(sqlFile)){
     dbTest <- RSQLite::dbConnect(RSQLite::SQLite(), dbname=sqlFile)
@@ -159,9 +160,9 @@ read.nodes<-function(nodeFile){
 #'  "9\t|\t32199\t|\tspecies\t|\tBA\t|\t0\t|\t1\t|\t11\t|\t1\t|\t0\t|\t1\t|\t1\t|\t0\t|\t\t|"
 #' )
 #' tmpFile<-tempfile()
-#' outFile<-tempfile()
+#' sqlFile<-tempfile()
 #' writeLines(nodes,tmpFile)
-#' read.nodes.sql(tmpFile,outFile)
+#' read.nodes.sql(tmpFile,sqlFile)
 read.nodes.sql<-function(nodeFile,sqlFile='nameNode.sqlite',overwrite=FALSE){
   if(file.exists(sqlFile)){
     dbTest <- RSQLite::dbConnect(RSQLite::SQLite(), dbname=sqlFile)
@@ -288,10 +289,10 @@ trimTaxa<-function(inFile,outFile,desiredCols=c(2,3)){
 #'   "Z17430\tZ17430.1\t3702\t16572"
 #' )
 #' inFile<-tempfile()
-#' outFile<-tempfile()
+#' sqlFile<-tempfile()
 #' writeLines(taxa,inFile)
-#' read.accession2taxid(inFile,outFile,vocal=FALSE)
-#' db<-RSQLite::dbConnect(RSQLite::SQLite(),dbname=outFile)
+#' read.accession2taxid(inFile,sqlFile,vocal=FALSE)
+#' db<-RSQLite::dbConnect(RSQLite::SQLite(),dbname=sqlFile)
 #' RSQLite::dbGetQuery(db,'SELECT * FROM accessionTaxa')
 #' RSQLite::dbDisconnect(db)
 read.accession2taxid<-function(taxaFiles,sqlFile,vocal=TRUE,extraSqlCommand='',indexTaxa=FALSE,overwrite=FALSE){
@@ -314,7 +315,7 @@ read.accession2taxid<-function(taxaFiles,sqlFile,vocal=TRUE,extraSqlCommand='',i
     trimTaxa(ii,tmp,1:3)
   }
   db <- RSQLite::dbConnect(RSQLite::SQLite(), dbname=sqlFile)
-  on.exit(RSQLite::dbDisconnect(db))
+  on.exit(RSQLite::dbDisconnect(db),add=TRUE)
   if(extraSqlCommand!='')RSQLite::dbExecute(db,extraSqlCommand)
   if(vocal)message('Reading in values. This may take a while.')
   RSQLite::dbWriteTable(conn = db, name = "accessionTaxa", value =tmp, row.names = FALSE, header = TRUE,sep='\t')
@@ -452,7 +453,7 @@ getParentNodes<-function(ids,sqlFile='nameNode.sqlite'){
   on.exit(RSQLite::dbDisconnect(db),add=TRUE)
   RSQLite::dbExecute(db, sprintf("ATTACH '%s' AS tmp",tmp))
   taxaDf<-RSQLite::dbGetQuery(db,'SELECT tmp.query.id, name,parent, rank FROM tmp.query LEFT OUTER JOIN nodes ON tmp.query.id=nodes.id LEFT OUTER JOIN names ON tmp.query.id=names.id WHERE names.scientific=1 OR names.scientific IS NULL')
-  if(!identical(taxaDf$id,ids))stop(simpleError('Problem finding ids'))
+  if(!identical(taxaDf$id,unname(ids)))stop(simpleError('Problem finding ids')) #don't actually need the unname here since as.numeric strips names but good to be safe
   return(taxaDf[,c('name','parent','rank')])
 }
 
@@ -605,7 +606,7 @@ accessionToTaxa<-function(accessions,sqlFile,version=c('version','base')){
   RSQLite::dbExecute(db,'DROP TABLE tmp.query')
   RSQLite::dbExecute(db,'DETACH tmp')
   file.remove(tmp)
-  if(!identical(taxaDf$accession,accessions))stop(simpleError('Query and SQL mismatch'))
+  if(!identical(taxaDf$accession,unname(accessions)))stop(simpleError('Query and SQL mismatch'))
   return(taxaDf$taxa)
 }
 
@@ -668,6 +669,7 @@ condenseTaxa<-function(taxaTable,groupings=rep(1,nrow(taxaTable))){
 #' @param outDir the directory to put names.dmp and nodes.dmp in
 #' @param url the url where taxdump.tar.gz is located
 #' @param fileNames the filenames desired from the tar.gz file
+#' @param timeout time in seconds for the download to time out
 #' @return a vector of file path strings of the locations of the output files
 #' @seealso \code{\link{read.nodes.sql}}, \code{\link{read.names.sql}}
 #' @references \url{ftp://ftp.ncbi.nih.gov/pub/taxonomy/}, \url{https://www.ncbi.nlm.nih.gov/Taxonomy/taxonomyhome.html/}
@@ -676,7 +678,10 @@ condenseTaxa<-function(taxaTable,groupings=rep(1,nrow(taxaTable))){
 #' \dontrun{
 #'   getNamesAndNodes()
 #' }
-getNamesAndNodes<-function(outDir='.',url='ftp://ftp.ncbi.nih.gov/pub/taxonomy/taxdump.tar.gz',fileNames=c('names.dmp','nodes.dmp')){
+getNamesAndNodes<-function(outDir='.',url='ftp://ftp.ncbi.nih.gov/pub/taxonomy/taxdump.tar.gz',fileNames=c('names.dmp','nodes.dmp'),timeout=36000){
+  oldTimeout<-getOption('timeout')
+  on.exit(options('timeout'=oldTimeout))
+  options('timeout'=timeout)
   outFiles<-file.path(outDir,fileNames)
   if(all(file.exists(outFiles))){
     message(paste(outFiles,collapse=', '),' already exist. Delete to redownload')
@@ -687,7 +692,7 @@ getNamesAndNodes<-function(outDir='.',url='ftp://ftp.ncbi.nih.gov/pub/taxonomy/t
   dir.create(tmp)
   tarFile<-file.path(tmp,base)
   utils::download.file(url,tarFile,mode='wb')
-  utils::untar(tarFile,fileNames,exdir=tmp,tar='internal',compressed='gzip')
+  utils::untar(tarFile,fileNames,exdir=tmp,tar='internal')
   tmpFiles<-file.path(tmp,fileNames)
   if(!all(file.exists(tmpFiles)))stop("Problem finding files ",paste(tmpFiles[!file.exists(tmpFiles)],collapse=', '))
   mapply(file.copy,tmpFiles,outFiles)
@@ -703,6 +708,7 @@ getNamesAndNodes<-function(outDir='.',url='ftp://ftp.ncbi.nih.gov/pub/taxonomy/t
 #' @param outDir the directory to put the accession2taxid.gz files in
 #' @param baseUrl the url of the directory where accession2taxid.gz files are located
 #' @param types the types if accession2taxid.gz files desired where type is the prefix of xxx.accession2taxid.gz. The default is to download all nucl_ accessions. For protein accessions, try \code{types=c('prot')}.
+#' @param timeout time in seconds for the download to time out
 #' @return a vector of file path strings of the locations of the output files
 #' @seealso \code{\link{read.accession2taxid}}
 #' @references \url{ftp://ftp.ncbi.nih.gov/pub/taxonomy/}, \url{https://www.ncbi.nlm.nih.gov/Sequin/acc.html}
@@ -717,7 +723,10 @@ getNamesAndNodes<-function(outDir='.',url='ftp://ftp.ncbi.nih.gov/pub/taxonomy/t
 #'
 #'   getAccession2taxid()
 #' }
-getAccession2taxid<-function(outDir='.',baseUrl='ftp://ftp.ncbi.nih.gov/pub/taxonomy/accession2taxid/',types=c('nucl_gb','nucl_wgs')){
+getAccession2taxid<-function(outDir='.',baseUrl='ftp://ftp.ncbi.nih.gov/pub/taxonomy/accession2taxid/',types=c('nucl_gb','nucl_wgs'),timeout=36000){
+  oldTimeout<-getOption('timeout')
+  on.exit(options('timeout'=oldTimeout))
+  options('timeout'=timeout)
   message('This can be a big (several gigabytes) download. Please be patient and use a fast connection.')
   fileNames<-sprintf('%s.accession2taxid.gz',types)
   outFiles<-file.path(outDir,fileNames)
@@ -793,10 +802,11 @@ getId2<-function(taxa,taxaNames){
 #' )
 #' tmpFile<-tempfile()
 #' writeLines(namesText,tmpFile)
-#' names<-read.names.sql(tmpFile)
-#' getId('Bacteria',names)
-#' getId('Not a real name',names)
-#' getId('Multi',names)
+#' sqlFile<-tempfile()
+#' read.names.sql(tmpFile,sqlFile)
+#' getId('Bacteria',sqlFile)
+#' getId('Not a real name',sqlFile)
+#' getId('Multi',sqlFile)
 getId<-function(taxa,sqlFile='nameNode.sqlite',onlyScientific=TRUE){
   if('data.table' %in% class(sqlFile))return(getId2(taxa,sqlFile))
   tmp<-tempfile()
@@ -883,10 +893,10 @@ prepareDatabase<-function(sqlFile='nameNode.sqlite',tmpDir='.',vocal=TRUE,...){
 #'   "Z17430\tZ17430.1\t3702\t16572"
 #' )
 #' inFile<-tempfile()
-#' outFile<-tempfile()
+#' sqlFile<-tempfile()
 #' writeLines(taxa,inFile)
-#' read.accession2taxid(inFile,outFile)
-#' getAccessions(3702,outFile)
+#' read.accession2taxid(inFile,sqlFile,vocal=FALSE)
+#' getAccessions(3702,sqlFile)
 getAccessions<-function(taxaId,sqlFile,version=c('version','base'),limit=NULL){
   version<-match.arg(version)
   if(version=='version')version<-'accession'
@@ -896,21 +906,40 @@ getAccessions<-function(taxaId,sqlFile,version=c('version','base'),limit=NULL){
   #set up a new table of accessions in a temp db (avoiding concurrency issues)
   #some trouble with dbWriteTable writing to "tmp.xxx" in the main database if we do this inside the attach
   tmpDb <- RSQLite::dbConnect(RSQLite::SQLite(), dbname=tmp)
+  on.exit(RSQLite::dbDisconnect(tmpDb))
   RSQLite::dbWriteTable(tmpDb,'query',data.frame('taxa'=taxaId,stringsAsFactors=FALSE),overwrite=TRUE)
-  RSQLite::dbDisconnect(tmpDb)
   #load the big sql
   db <- RSQLite::dbConnect(RSQLite::SQLite(), dbname=sqlFile)
+  on.exit(RSQLite::dbDisconnect(db),add=TRUE)
   #attach the temp table
   RSQLite::dbExecute(db, sprintf("ATTACH '%s' AS tmp",tmp))
   taxaDf<-RSQLite::dbGetQuery(db,sprintf('SELECT tmp.query.taxa, %s FROM tmp.query LEFT OUTER JOIN accessionTaxa ON tmp.query.taxa=accessionTaxa.taxa%s',version,ifelse(!is.null(limit),sprintf(' LIMIT %s',limit),'')))
   RSQLite::dbExecute(db,'DROP TABLE tmp.query')
   RSQLite::dbExecute(db,'DETACH tmp')
-  RSQLite::dbDisconnect(db)
   file.remove(tmp)
   colnames(taxaDf)<-c('taxa','accession')
   return(taxaDf)
 }
 
+#' Create a Newick tree from taxonomy
+#'
+#' Create a Newick formatted tree from a data.frame of taxonomic assignments
+#' @param taxa a matrix with a row for each leaf of the tree and a column for each taxonomic classification e.g. the output from getTaxonomy
+#' @param naSub a character string to substitute in place of NAs in the taxonomy
+#' @seealso \code{\link{getTaxonomy}}
+#' @export
+#' @examples
+#' taxa<-matrix(c('A','A','A','B','B','C','D','D','E','F','G','H'),nrow=3)
+#' makeNewick(taxa)
+makeNewick<-function(taxa,naSub='_'){
+  if(!is.null(naSub))taxa[is.na(taxa)]<-naSub
+  if(ncol(taxa)==0)return('')
+  bases<-unique(taxa[,1])
+  innerTree<-sapply(bases,function(ii)makeNewick(taxa[taxa[,1]==ii,-1,drop=FALSE]))
+  out<-sprintf('(%s)',paste(sprintf('%s%s',innerTree,bases),collapse=','))
+  return(out)
+}
+
 #' Switch from data.table to SQLite
 #'
 #' In version 0.5.0, taxonomizr switched from data.table to SQLite name and node lookups. See below for more details.
